@@ -1,5 +1,5 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/display.c,v 1.1.1.1 2002/06/05 16:59:18 richb Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/display.c,v 1.3 2002/06/14 21:02:41 richb Exp $
  *
  *  Copyright (c) 1987-2002, Sun Microsystems, Inc.  All Rights Reserved.
  *  Sun considers its source code as an unpublished, proprietary
@@ -20,7 +20,6 @@
 
 #include <stdio.h>
 #include <string.h>
-#include "color.h"
 #include "calctool.h"
 #include "extern.h"
 
@@ -175,7 +174,7 @@ make_number(int *MPnumber)     /* Convert MP number to character string. */
     double number, val;
 
 /*  NOTE: make_number can currently set v->error when converting to a double.
- *        This is to provide the same look&feel as V3 even though calctool
+ *        This is to provide the same look&feel as V3 even though gcalctool
  *        now does internal arithmetic to "infinite" precision.
  *
  *  XXX:  Needs to be improved. Shouldn't need to convert to a double in
