@@ -1,5 +1,5 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/functions.c,v 1.1.1.1 2002/06/05 16:59:18 richb Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/functions.c,v 1.3 2002/06/20 22:04:24 richb Exp $
  *
  *  Copyright (c) 1987-2002, Sun Microsystems, Inc.  All Rights Reserved.
  *  Sun considers its source code as an unpublished, proprietary
@@ -22,7 +22,6 @@
 #include <stdlib.h>
 #include <errno.h>
 #include <string.h>
-#include "color.h"
 #include "calctool.h"
 #include "extern.h"
 
@@ -542,41 +541,10 @@ do_function()      /* Perform a user defined function. */
 
 
 void
-do_help()            /* F1 key was pressed; show help message. */
+do_help()            /* Help key was pressed; show help message. */
 {
-  fprintf(stderr, "do_help called.\n") ;
-#ifdef FIXUP
-    char help_str[MAXLINE];  /* Passed to the help display routine. */
-    int column, n, row;
-
-    SPRINTF(help_str, "%s:", "calctool");
-    if (v->cury <= v->ndisplay && 
-        v->curwin == FCP_KEY) {             /* Help over display? */
-        STRCAT(help_str, "DISP");
-        show_help(help_str);
-    } else {                    /* Check for help over buttons. */
-        if (v->curwin == FCP_KEY) {
-            v->cury -= v->ndisplay;
-        }
-        for (row = 0; row < MAXROWS; row++) {
-            for (column = 0; column < MAXCOLS; column++) {
-                if ((v->curx > (column * (v->bwidth + v->bgap) + v->bborder)) &&
-                    (v->curx < (column * (v->bwidth + v->bgap) + v->bborder +
-                                                                 v->bwidth)) &&
-                    (v->cury > (row * (v->bheight + v->bgap) + v->bborder)) &&
-                    (v->cury < (row * (v->bheight + v->bgap) + v->bborder +
-                                                               v->bheight))) {
-                    n = row * MAXCOLS + column;
-                    STRCAT(help_str, help_button_str(row * MAXCOLS + column));
-                    show_help(help_str);
-                    return;
-                }
-            }
-        }
-        STRCAT(help_str, "canvas");
-        show_help(help_str);
-    }
-#endif /*FIXUP*/
+    v->show_help = !v->show_help;
+    set_help_state(v->show_help); 
 }
 
 
@@ -791,7 +759,7 @@ do_paren()
             while (*ptr != '(') {
                 ptr++;
             }
-            while (*ptr != NULL) {
+            while (*ptr) {
                 process_parens(*ptr++);
             }
             return;
