@@ -1,5 +1,5 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/gtk.c,v 1.1.1.1 2002/06/05 16:59:18 richb Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/gtk.c,v 1.10 2002/06/24 22:07:09 richb Exp $
  *
  *  Copyright (c) 1987-2002, Sun Microsystems, Inc.  All Rights Reserved.
  *  Sun considers its source code as an unpublished, proprietary
@@ -24,7 +24,7 @@
 #include <sys/param.h>
 #include <sys/stat.h>
 #include <netdb.h>
-#include "color.h"
+#include <pwd.h>
 #include "calctool.h"
 #include "config.h"
 #include "extern.h"
@@ -33,7 +33,6 @@
 #include <gdk/gdkx.h>
 #include <gdk/gdkkeysyms.h>
 #include <X11/Xresource.h>
-#include "display_frame.h"
 
 #define BUT_0     X->buttons[v->righthand ? 44 : 40]       /* 0 */
 #define BUT_1     X->buttons[v->righthand ? 36 : 32]       /* 1 */
@@ -65,9 +64,8 @@ typedef struct Xobject {               /* Gtk+/Xlib graphics object. */
     GtkAccelGroup *kbd_accel;
     GtkAccelGroup *menu_accel;
     GdkAtom clipboard_atom;
-    GdkColor palette[CALC_COLORSIZE];  /* Gtk+ color palette. */
-    GdkColormap *cmap;
     GtkItemFactory *fact[MAXMENUS];
+    GtkTooltips *tips;
     GtkWidget *buttons[NOBUTTONS];
     GtkWidget *mode_buttons[(MROWS * MCOLS) * (MAXMODES-1)];
     GtkWidget *aframe;                 /* ASCII window. */
@@ -84,8 +82,6 @@ typedef struct Xobject {               /* Gtk+/Xlib graphics object. */
     GtkWidget *mframe;                 /* Mode window. */
     GtkWidget *labels[MAXLABELS];      /* Entry and mode/state labels. */
     GtkWidget *pframe;                 /* Properties window. */
-    GtkWidget *pdcolor;
-    GtkWidget *pdmono;
     GtkWidget *psleft;
     GtkWidget *psright;
     GtkWidget *rframe;                 /* Register window. */
@@ -94,17 +90,54 @@ typedef struct Xobject {               /* Gtk+/Xlib graphics object. */
     GtkWidget *mode_tables[MAXMODES];
 
     enum menu_type CFtype;
-    Display *dpy;                      /* Display ids of calctool frames. */
+    Display *dpy;
     XrmDatabase rDB;                   /* Combined resources database. */
 
-    int cmap_loaded;              /* Has the colormap being loaded? */
-    int depth;                    /* Depth of the visual we are using. */
     int menuval;                  /* Index to button array at menu time. */
     int mrec[MAXMENUS];
 } XObject;
 
 typedef struct Xobject *XVars;
 
+static char *display_help = _( \
+    "Calculator display\n" \
+    "\n" \
+    "The bottom of the display shows the current mode\n" \
+    "of operation for:\n" \
+    "\n" \
+    "Number base         BIN (binary),\n" \
+    "                    OCT (octal),\n" \
+    "                    DEC (decimal),\n" \
+    "                    or HEX (hexadecimal)\n" \
+    "\n" \
+    "Trigonometric base  DEG (degrees),\n" \
+    "                    GRAD (gradients),\n" \
+    "                    or RAD (radians)\n" \
+    "\n" \
+    "Display type        ENG (engineering),\n" \
+    "                    FIX (fixed point),\n" \
+    "                    or SCI (scientific)\n" \
+    "\n" \
+    "Mode                BASIC, FINANCIAL,\n" \
+    "                    LOGICAL, or SCIENTIFIC\n" \
+    "\n" \
+    "At certain times, the following may also be\n" \
+    "displayed:\n" \
+    "\n" \
+    "HYP     (hyperbolic mode is set)\n" \
+    "INV     (inverse mode is set)\n" \
+    "\n" \
+    "These pertain only to trigonometrical\n" \
+    "calculations.\n" \
+    "\n" \
+    "When the calculator needs to be \"cleared,\"\n" \
+    "\"CLR\" will be displayed.\n" \
+    "\n" \
+    "The display also shows the current mathematical\n" \
+    "operation and function when this operation\n" \
+    "requires other input to be complete."
+    );
+
 static GtkWidget *create_menu(enum menu_type, int);
 static GtkWidget *make_menu_button(gchar *, int);
 static GtkWidget *make_mtable(GtkWidget *, GtkWidget *, enum mode_type);
@@ -119,7 +152,6 @@ static gboolean dismiss_pframe(GtkWidget *, GdkEvent *, gpointer);
 static gboolean dismiss_rframe(GtkWidget *, GdkEvent *, gpointer);
 static gboolean frame_interpose(GtkWidget *, GdkEvent *, gpointer );
 
-static void load_colors(void);
 static void aframe_apply_cb(GtkButton *, gpointer);
 static void aframe_cancel_cb(GtkButton *, gpointer);
 static void aframe_entry_cb(GtkEntry *, gpointer);
@@ -141,6 +173,7 @@ static void prop_apply(GtkButton *, gpointer);
 static void prop_cancel(GtkButton *, gpointer);
 static void prop_ok(GtkButton *, gpointer);
 static void props_menu_proc(gpointer, int, GtkWidget *);
+static void remove_kbd_accel(GtkWidget *, guint, guint);
 static void reset_prop_vals(void);
 static void set_button_label(GtkWidget *, gchar *, int n);
 static void set_prop_options(int);
@@ -270,6 +303,9 @@ static GtkItemFactoryEntry props_menu[] = {
 int
 main(int argc, char **argv)
 {
+    char name[MAXLINE];          /* Full name of users .gcalctoolrc file. */
+    struct passwd *entry;
+
     v = (Vars)  LINT_CAST(calloc(1, sizeof(CalcVars)));
     X = (XVars) LINT_CAST(calloc(1, sizeof(XObject)));
 
@@ -278,13 +314,21 @@ main(int argc, char **argv)
 
     gtk_set_locale();
     gtk_init(&argc, &argv);
+
+    gtk_rc_get_default_files();
+    if ((v->home = getenv("HOME")) == NULL) {
+        if ((entry = getpwuid(getuid())) != NULL) {
+            v->home = entry->pw_dir;
+        }
+    }
+    SPRINTF(name, "%s/%s", v->home, RCNAME);
+    gtk_rc_parse(name);
+
     X->kbd_accel = gtk_accel_group_new();
     X->menu_accel = gtk_accel_group_new();
+    X->tips = gtk_tooltips_new();
     X->dpy = GDK_DISPLAY();
 
-/* XXX: remove when working well. */
-/*    XSynchronize(X->dpy, TRUE); */
-
     do_calctool(argc, argv);
        
     return(0);
@@ -496,75 +540,6 @@ cfframe_ok_cb(GtkButton *button, gpointer user_data)
 }
 
 
-/*  Checks for font, width, height, scale, header, geometry, icon image
- *  icon label, reverse video, foreground color and background color.
- */
-
-void
-check_args()
-{
-#ifdef FIXUP
-    char *tempstr;
-    int reverse = 0;
-
-    if (defaults_exists("window.width", "Window.Width")) {
-        v->iwidth = defaults_get_integer("window.width", "Window.Width", 1);
-        if (v->iwidth < 0) {
-            v->iwidth = -1;
-        }
-    }
-    if (defaults_exists("window.height", "Window.Height")) {
-        v->iheight = defaults_get_integer("window.height", "Window.Height", 1);
-        if (v->iheight < 0) {
-            v->iheight = -1;
-        }
-    }
-
-    if (defaults_exists("window.iconic", "Window.Iconic")) {
-        v->iconic = defaults_get_boolean("window.iconic", "Window.Iconic", 0);
-    }
-
-    if (defaults_exists("window.header", "Window.Header")) {
-        tempstr = (char *) defaults_get_string("window.header",
-                                               "Window.Header", "Calculator");
-        if (tempstr == NULL || *tempstr == '\0') {
-            tempstr = " ";
-        }
-        read_str(&v->titleline, tempstr);
-    }
-    if (defaults_exists("window.geometry", "Window.Geometry")) {
-        tempstr = (char *) defaults_get_string("window.geometry",
-                                               "Window.Geometry", "");
-        ds_get_geometry_size(tempstr, &v->iwidth, &v->iheight);
-    }
-    if (defaults_exists("icon.pixmap", "Icon.Pixmap")) {
-        tempstr = (char *) defaults_get_string("icon.pixmap",
-                                               "Icon.Pixmap", "");
-        if (tempstr != NULL && *tempstr != '\0' && access(tempstr, R_OK) == 0) {
-            v->hasicon = TRUE;
-        } else {
-            FPRINTF(stderr, _("%s: cannot read icon filename (%s)\n"), 
-                    v->progname, tempstr);
-            xv_usage(_("calctool"));
-            exit(1);
-        }
-    }
-    if (defaults_exists("icon.footer", "Icon.Footer")) {
-        tempstr = (char *) defaults_get_string("icon.footer", "Icon.Footer",
-                                               _("Constant no:"));
-        if (tempstr == NULL || *tempstr == '\0') {
-            tempstr = " ";
-        }
-        read_str(&v->iconlabel, tempstr);
-    }
-    if (defaults_exists("window.reverseVideo", "Window.ReverseVideo")) {
-        reverse = defaults_get_boolean("window.reverseVideo",
-                                       "Window.ReverseVideo", 0);
-    }
-#endif /*FIXUP*/
-}
-
-
 /*ARGSUSED*/
 static void
 con_menu_proc(gpointer data, int choice, GtkWidget *item)
@@ -639,77 +614,68 @@ create_aframe()  /* Create auxiliary frame for ASC key. */
 static void
 create_cfframe()    /* Create auxiliary frame for CON/FUN key. */
 {
-    GtkWidget *vbox2, *hbox2, *hbox3, *cf_desc_label, *hbox4, *cf_val_label; 
-    GtkWidget *hbox5, *cf_ok, *cf_apply, *cf_cancel;
+    GtkWidget *cf_desc_label, *cf_val_label, *cf_ok, *cf_apply, *cf_cancel;
+    GtkWidget *hbox, *table, *vbox;
  
     X->cfframe = gtk_window_new(GTK_WINDOW_TOPLEVEL);
     gtk_window_set_title(GTK_WINDOW(X->cfframe), _("New Constant"));
  
-    vbox2 = gtk_vbox_new(FALSE, 0);
-    gtk_widget_show(vbox2);
-    gtk_container_add(GTK_CONTAINER(X->cfframe), vbox2);
- 
-    hbox2 = gtk_hbox_new(FALSE, 0);
-    gtk_widget_show(hbox2);
-    gtk_box_pack_start(GTK_BOX(vbox2), hbox2, TRUE, TRUE, 0);
+    vbox = gtk_vbox_new(FALSE, 0);
+    gtk_container_add(GTK_CONTAINER(X->cfframe), vbox);
  
+    table = gtk_table_new(3, 2, FALSE);
+    gtk_widget_ref(table);
+    gtk_box_pack_start(GTK_BOX(vbox), table, TRUE, TRUE, 0);
+
     X->cf_no_label = gtk_label_new(_("Constant no:"));
-    gtk_widget_show(X->cf_no_label);
-    gtk_box_pack_start(GTK_BOX(hbox2), X->cf_no_label, FALSE, FALSE, 0);
-    gtk_label_set_justify(GTK_LABEL(X->cf_no_label), GTK_JUSTIFY_RIGHT);
-    gtk_misc_set_padding(GTK_MISC(X->cf_no_label), 5, 0);
+    gtk_misc_set_alignment(GTK_MISC(X->cf_no_label), 1.0, 0.5);
+    gtk_table_attach(GTK_TABLE(table), X->cf_no_label, 0, 1, 0, 1,
+                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
+                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 4, 4);
  
     X->cf_con_entry = gtk_entry_new();
     gtk_entry_set_max_length(GTK_ENTRY(X->cf_con_entry), 2);
-    gtk_widget_show(X->cf_con_entry);
-    gtk_box_pack_start(GTK_BOX(hbox2), X->cf_con_entry, TRUE, TRUE, 0);
- 
-    hbox3 = gtk_hbox_new(FALSE, 0);
-    gtk_widget_show(hbox3);
-    gtk_box_pack_start(GTK_BOX(vbox2), hbox3, TRUE, TRUE, 0);
+    gtk_table_attach(GTK_TABLE(table), X->cf_con_entry, 1, 2, 0, 1,
+                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 
+                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 4, 4); 
  
     cf_desc_label = gtk_label_new(_("Description:"));
-    gtk_widget_show(cf_desc_label);
-    gtk_box_pack_start(GTK_BOX(hbox3), cf_desc_label, FALSE, FALSE, 0);
-    gtk_label_set_justify(GTK_LABEL(cf_desc_label), GTK_JUSTIFY_RIGHT);
-    gtk_misc_set_padding(GTK_MISC(cf_desc_label), 5, 0);
+    gtk_misc_set_alignment(GTK_MISC(cf_desc_label), 1.0, 0.5);
+    gtk_table_attach(GTK_TABLE(table), cf_desc_label, 0, 1, 1, 2,
+                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),  
+                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 4, 4);
  
     X->cf_desc_entry = gtk_entry_new();
     gtk_entry_set_max_length(GTK_ENTRY(X->cf_desc_entry), 30);
-    gtk_widget_show(X->cf_desc_entry);
-    gtk_box_pack_start(GTK_BOX(hbox3), X->cf_desc_entry, FALSE, FALSE, 0);
+    gtk_table_attach(GTK_TABLE(table), X->cf_desc_entry, 1, 2, 1, 2, 
+                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),   
+                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 4, 4); 
  
-    hbox4 = gtk_hbox_new(FALSE, 0);
-    gtk_widget_show(hbox4);
-    gtk_box_pack_start(GTK_BOX(vbox2), hbox4, TRUE, TRUE, 0);
-
     cf_val_label = gtk_label_new(_("Value:"));
-    gtk_widget_show(cf_val_label);
-    gtk_box_pack_start(GTK_BOX(hbox4), cf_val_label, FALSE, FALSE, 0);
-    gtk_label_set_justify(GTK_LABEL(cf_val_label), GTK_JUSTIFY_RIGHT);
+    gtk_misc_set_alignment(GTK_MISC(cf_val_label), 1.0, 0.5);
+    gtk_table_attach(GTK_TABLE(table), cf_val_label, 0, 1, 2, 3, 
+                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),   
+                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 4, 4); 
 
     X->cf_val_entry = gtk_entry_new();
     gtk_entry_set_max_length(GTK_ENTRY(X->cf_val_entry), 30);
-    gtk_widget_show(X->cf_val_entry);
-    gtk_box_pack_start(GTK_BOX(hbox4), X->cf_val_entry, TRUE, TRUE, 0);
+    gtk_table_attach(GTK_TABLE(table), X->cf_val_entry, 1, 2, 2, 3,            
+                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),    
+                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 4, 4); 
 
-    hbox5 = gtk_hbox_new(FALSE, 0);
-    gtk_widget_show(hbox5);
-    gtk_box_pack_start(GTK_BOX (vbox2), hbox5, TRUE, TRUE, 0);
+    hbox = gtk_hbox_new(FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
 
     cf_ok = gtk_button_new_with_label(_("OK"));
-    gtk_widget_show(cf_ok);
-    gtk_box_pack_start(GTK_BOX (hbox5), cf_ok, TRUE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(hbox), cf_ok, TRUE, TRUE, 0);
     gtk_container_set_border_width(GTK_CONTAINER (cf_ok), 2);
 
     cf_apply = gtk_button_new_with_label(_("Apply"));
-    gtk_widget_show(cf_apply);
-    gtk_box_pack_start(GTK_BOX(hbox5), cf_apply, TRUE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(hbox), cf_apply, TRUE, TRUE, 0);
     gtk_container_set_border_width(GTK_CONTAINER(cf_apply), 2);
 
     cf_cancel = gtk_button_new_with_label(_("Cancel"));
-    gtk_widget_show(cf_cancel);
-    gtk_box_pack_start(GTK_BOX(hbox5), cf_cancel, TRUE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(hbox), cf_cancel, TRUE, TRUE, 0);
     gtk_container_set_border_width(GTK_CONTAINER(cf_cancel), 2);
 
     g_signal_connect(G_OBJECT(X->cfframe), "delete_event",
@@ -720,6 +686,7 @@ create_cfframe()    /* Create auxiliary frame for CON/FUN key. */
                      G_CALLBACK(cfframe_apply_cb), NULL);
     g_signal_connect(G_OBJECT(cf_cancel), "clicked",
                      G_CALLBACK(cfframe_cancel_cb), NULL);
+    gtk_widget_show_all(vbox);
     gtk_widget_realize(X->cfframe);
 }
 
@@ -734,11 +701,9 @@ create_kbd_accel(GtkWidget *button, guint button_mods, guint button_key) {
 void
 create_kframe()
 {
-    char *tool_label = NULL, *hn;
-    char data[12];
+    char *tool_label = NULL, *hn, name[MAXLINE];
     int i;
-    GtkStyle *style;
-    GtkWidget *frame, *hbox, *vbox;
+    GtkWidget *event_box, *hbox;
 
     if (v->titleline == NULL) {
         hn = make_hostname(X->dpy);
@@ -758,22 +723,17 @@ create_kframe()
     gtk_widget_ref(X->kvbox);
     gtk_container_add(GTK_CONTAINER(X->kframe), X->kvbox);
 
-    frame = calctool_display_frame_new();
-    gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
-    gtk_widget_modify_bg(frame, GTK_STATE_NORMAL, &X->palette[C_WHITE]);
-
-    vbox = gtk_vbox_new(FALSE, 0);
-    gtk_container_add(GTK_CONTAINER(frame), vbox);
-
+    event_box = gtk_event_box_new();
     X->labels[(int) DISPLAYITEM] = gtk_label_new("");
+    SPRINTF(name, "label%1d", (int) DISPLAYITEM);
+    gtk_widget_set_name(X->labels[(int) DISPLAYITEM], name);
+    gtk_tooltips_set_tip(X->tips, event_box, display_help, "");
 
     gtk_label_set_selectable(GTK_LABEL(X->labels[(int) DISPLAYITEM]), TRUE);
     set_label(DISPLAYITEM, "0.00");
     gtk_widget_ref(X->labels[(int) DISPLAYITEM]);
-
-    gtk_box_pack_start(GTK_BOX(X->kvbox), frame, TRUE, TRUE, 0);
-    gtk_box_pack_start(GTK_BOX(vbox), X->labels[(int) DISPLAYITEM], 
-                       TRUE, TRUE, 0);
+    gtk_container_add(GTK_CONTAINER(event_box), X->labels[(int) DISPLAYITEM]);
+    gtk_box_pack_start(GTK_BOX(X->kvbox), event_box, TRUE, TRUE, 0);
 
     gtk_misc_set_alignment(GTK_MISC(X->labels[(int) DISPLAYITEM]), 1.0, 0.5);
     gtk_misc_set_padding(GTK_MISC(X->labels[(int) DISPLAYITEM]), 5, 5);
@@ -791,6 +751,9 @@ create_kframe()
             continue;                   /* DISPLAYITEM is already created. */
         }
         X->labels[i] = gtk_label_new("");
+        SPRINTF(name, "label%1d", i);
+        gtk_widget_set_name(X->labels[i], name);
+
         switch (i) {
             case BASEITEM :
                 set_label(i, base_str[(int) v->base]);
@@ -815,14 +778,11 @@ create_kframe()
         }
 
         gtk_widget_ref(X->labels[i]);
-        SPRINTF(data, "label%1d", i);
         gtk_box_pack_start(GTK_BOX(hbox), X->labels[i], TRUE, TRUE, 0);
-        style = gtk_style_copy(gtk_widget_get_style(X->labels[i]));
-        gtk_widget_set_style(X->labels[i], style); 
         gtk_misc_set_padding(GTK_MISC(X->labels[i]), 5, 0);
     }
 
-    gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(X->kvbox), hbox, TRUE, TRUE, 0);
     make_ktable(X->kframe, X->kvbox);
     add_extra_kbd_accels();
     gtk_window_add_accel_group(GTK_WINDOW(X->kframe), X->kbd_accel);
@@ -866,8 +826,9 @@ void
 create_rframe()
 {
     char line[MAXLINE];     /* Current memory register line. */
+    char name[MAXLINE];
     int i;
-    GtkWidget *frame, *vbox;
+    GtkWidget *vbox;
 
     X->rframe = gtk_window_new(GTK_WINDOW_TOPLEVEL);
     g_object_set_data(G_OBJECT(X->rframe), "rframe", X->rframe);
@@ -875,26 +836,22 @@ create_rframe()
     set_title(FCP_REG, _("Memory Registers"));
     gtk_widget_set_size_request(X->rframe, 248, 179);
 
-    frame = calctool_display_frame_new();
-    gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
-    gtk_widget_modify_bg(frame, GTK_STATE_NORMAL, &X->palette[C_WHITE]);
-
     vbox = gtk_vbox_new(FALSE, 0);
-    gtk_container_add(GTK_CONTAINER(frame), vbox);
-
-    gtk_container_add(GTK_CONTAINER(X->rframe), frame);
+    gtk_container_add(GTK_CONTAINER(X->rframe), vbox);
     gtk_widget_realize(X->rframe);
 
     for (i = 0; i < MAXREGS; i++) {
         SPRINTF(line, "%1d   %s", i,  make_number(v->MPmvals[i]));
         X->regs[i] = gtk_label_new(line);
+        SPRINTF(name, "register_label%1d", i);
+        gtk_widget_set_name(X->regs[i], name);
         gtk_widget_ref(X->regs[i]);
         gtk_misc_set_alignment(GTK_MISC(X->regs[i]), 0.0, 0.5);
         gtk_misc_set_padding(GTK_MISC(X->regs[i]), 5, 5);
         gtk_widget_set_size_request(X->regs[i], -1, 80);
         gtk_box_pack_start(GTK_BOX(vbox), X->regs[i], TRUE, TRUE, 0);
     }
-    gtk_widget_show_all(frame);
+    gtk_widget_show_all(vbox);
 
     g_signal_connect(G_OBJECT(X->rframe), "delete_event",
                      G_CALLBACK(dismiss_rframe), NULL);
@@ -1007,7 +964,7 @@ display_prop_sheet()
 
 
 void
-do_keys()      /* Display/undisplay the calctool key values. */
+do_keys()      /* Display/undisplay the gcalctool key values. */
 {
     enum fcp_type curwin;
     enum mode_type modetype;
@@ -1140,6 +1097,29 @@ frame_interpose(GtkWidget *widget, GdkEvent *event, gpointer user_data)
             v->event_type = TAKE_FROM_SHELF;
         } else if ((key == GDK_Help) && down) {
             v->event_type = SHOWHELP;
+
+/* Hack Alert.
+ * There is a bug: http://bugzilla.gnome.org/show_bug.cgi?id=79184
+ * that is preventing the numeric keypad "+" and "." keys from working
+ * correctly. Until this is fixed, an explicit test for those two keys
+ * has been added here.
+ */
+        } else if (type == GDK_KEY_PRESS && key == GDK_KP_Decimal) {
+            process_item(v->righthand ? 45 : 41);
+            return(TRUE);
+        } else if (type == GDK_KEY_PRESS && key == GDK_KP_Add) {
+            process_item(v->righthand ? 47 : 43);
+            return(TRUE);
+        } else if (v->pending) {
+            if (key == GDK_Shift_L || key == GDK_Shift_R) {
+                return(FALSE);
+            }
+            if (type == GDK_KEY_PRESS) {
+                v->current = key;
+                do_pending();
+            } else {
+                return(FALSE);
+            }
         } else {
 
 /* If it's a not a special keyboard down event, then let the keyboard
@@ -1235,7 +1215,7 @@ get_proc(GtkClipboard *clipboard, const gchar *text, gpointer data)
 }
 
 
-/* Get calctool resource from merged database. */
+/* Get gcalctool resource from merged database. */
 
 char *
 get_resource(enum res_type rtype)
@@ -1282,48 +1262,6 @@ char *MSGFILE_LABEL   = "SUNW_DESKSET_CALCTOOL_LABEL";
 char *MSGFILE_MESSAGE = "SUNW_DESKSET_CALCTOOL_MSG";
 
 
-static void
-load_colors(void)      /* Create and load calctool color map. */
-{
-    GdkColor color;
-    int i, numcolors;
-
-    if (v->iscolor) {
-        numcolors  = 0;
-        for (i = 0; i < CALC_COLORSIZE; i++) {
-            if (v->colstr[i] == NULL ||
-                gdk_color_parse(v->colstr[i], &color) == FALSE) {
-                color.red   = (unsigned short) (v->rcols[i] << 8);
-                color.green = (unsigned short) (v->gcols[i] << 8);
-                color.blue  = (unsigned short) (v->bcols[i] << 8);
-            }
-            if (gdk_colormap_alloc_color(X->cmap, &color, TRUE, TRUE) == TRUE) {
-                X->palette[numcolors++] = color;
-            }
-        }
-        if (numcolors < CALC_COLORSIZE) {
-            FPRINTF(stderr, _("%s: cannot allocate colors. "), v->progname);
-            FPRINTF(stderr, _("Starting monochrome version.\n"));
-            v->iscolor = 0;
-        } else {
-            X->cmap_loaded = 1;
-        }
-    }
-}
-
-
-void
-load_deskset_defs()     /* Load current deskset resource database. */
-{
-#ifdef FIXUP
-    if (X->desksetDB != NULL) {
-        XrmDestroyDatabase(X->desksetDB);
-    }
-    X->desksetDB = ds_load_deskset_defs();
-#endif /*FIXUP*/
-}
-
-
 void
 load_resources()        /* Load combined X resources databases. */
 { 
@@ -1334,7 +1272,7 @@ load_resources()        /* Load combined X resources databases. */
         X->rDB = NULL;
         return;
     } else if ((ptr = getenv("CALCTOOLDEFAULTS")) == NULL) {
-        SPRINTF(name, "%s/.calctooldefaults", getenv("HOME"));
+        SPRINTF(name, "%s/.gcalctooldefaults", getenv("HOME"));
         ptr = name;
     }
     X->rDB = XrmGetFileDatabase(ptr);
@@ -1344,24 +1282,8 @@ load_resources()        /* Load combined X resources databases. */
 void
 make_frames()
 {
-    int dummy;
-    GdkVisual *visual;
-
-    X->cmap = gdk_colormap_get_system();
-    gdk_window_get_geometry(gdk_get_default_root_window(),
-                            &dummy, &dummy, &dummy, &dummy, &X->depth);
-    visual = gdk_visual_get_system();
-    if ((visual->depth > 1) &&
-        ((visual->type == GDK_VISUAL_PSEUDO_COLOR) ||
-         (visual->type == GDK_VISUAL_STATIC_COLOR) ||
-         (visual->type == GDK_VISUAL_TRUE_COLOR) ||
-         (visual->type == GDK_VISUAL_DIRECT_COLOR))) {
-        v->iscolor = 1;
-    }
-
     X->clipboard_atom = gdk_atom_intern("CLIPBOARD", FALSE);
-    load_colors();                       /* Load the calctool colormap. */
-    create_kframe();                     /* Create main calctool window. */
+    create_kframe();                     /* Create main gcalctool window. */
     create_mframe();                     /* Create mode window. */
     create_rframe();                     /* Create memory register window. */
     set_mode(v->modetype);
@@ -1406,8 +1328,8 @@ make_hostname(Display *dpy)
 static GtkWidget *
 make_mtable(GtkWidget *frame, GtkWidget *vbox, enum mode_type modetype)
 {
-    int i, j, k, n;
-    GtkStyle *style;
+    char name[MAXLINE];
+    int i, j, n;
     GtkWidget *table = gtk_table_new(MROWS, MCOLS, FALSE);
  
     gtk_widget_ref(table);
@@ -1420,26 +1342,26 @@ make_mtable(GtkWidget *frame, GtkWidget *vbox, enum mode_type modetype)
             if (mode_buttons[n].mtype == M_NONE) {
                 X->mode_buttons[n] = 
                            gtk_button_new_with_label(mode_buttons[n].str);
-                g_signal_connect(G_OBJECT(X->mode_buttons[n]), "clicked",
-                                 G_CALLBACK(button_proc), 
-                                 (gpointer) (j*MCOLS + i));
             } else {
                 X->mode_buttons[n] = make_menu_button(mode_buttons[n].str,
                                                       j*MCOLS + i);
             }
+            g_signal_connect(G_OBJECT(X->mode_buttons[n]), "clicked",
+                             G_CALLBACK(button_proc), (gpointer) (j*MCOLS + i));
+            SPRINTF(name, "mode_button%1d", n);
+            gtk_widget_set_name(X->mode_buttons[n], name);
+            gtk_tooltips_set_tip(X->tips, X->mode_buttons[n],
+                                 mode_buttons[n].hstr, "");
             g_object_set_data(G_OBJECT(X->mode_buttons[n]), "frame", X->mframe);
             gtk_widget_ref(X->mode_buttons[n]);
-            style = gtk_style_copy(gtk_widget_get_style(X->mode_buttons[n]));
-            for (k = 0; k < 5; k++) {
-                style->bg[k] = X->palette[(int) mode_buttons[n].color];
-            }   
-            gtk_widget_set_style(X->mode_buttons[n], style);
 
             if (strcmp(mode_buttons[n].str, "    ")) {
                 create_kbd_accel(X->mode_buttons[n], mode_buttons[n].mods,
                                  mode_buttons[n].value);
-                gtk_widget_show(X->mode_buttons[n]);
+            } else {
+                gtk_widget_set_sensitive(X->mode_buttons[n], FALSE);
             }
+            gtk_widget_show(X->mode_buttons[n]);
             gtk_table_attach(GTK_TABLE(table), X->mode_buttons[n],
                              i, i+1, j, j+1,
                              (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
@@ -1455,8 +1377,8 @@ make_mtable(GtkWidget *frame, GtkWidget *vbox, enum mode_type modetype)
 static void
 make_ktable(GtkWidget *frame, GtkWidget *vbox)
 {
-    int i, j, k, n;
-    GtkStyle *style;
+    char name[MAXLINE];
+    int i, j, n;
 
     X->ktable = gtk_table_new(BROWS, BCOLS, FALSE);
     gtk_widget_ref(X->ktable);
@@ -1468,18 +1390,16 @@ make_ktable(GtkWidget *frame, GtkWidget *vbox)
             n = j*BCOLS + i;
             if (buttons[n].mtype == M_NONE) {
                 X->buttons[n] = gtk_button_new_with_label(buttons[n].str);
-                g_signal_connect(G_OBJECT(X->buttons[n]), "clicked",
-                                 G_CALLBACK(button_proc), (gpointer) n);
             } else {
                 X->buttons[n] = make_menu_button(buttons[n].str, n);
             }   
+            g_signal_connect(G_OBJECT(X->buttons[n]), "clicked",
+                             G_CALLBACK(button_proc), (gpointer) n);
+            SPRINTF(name, "button%1d", n);
+            gtk_widget_set_name(X->buttons[n], name);
+            gtk_tooltips_set_tip(X->tips, X->buttons[n], buttons[n].hstr, "");
             g_object_set_data(G_OBJECT(X->buttons[n]), "frame", X->kframe);
             gtk_widget_ref(X->buttons[n]);
-            style = gtk_style_copy(gtk_widget_get_style(X->buttons[n]));
-            for (k = 0; k < 5; k++) {
-                style->bg[k] = X->palette[(int) buttons[n].color];
-            }   
-            gtk_widget_set_style(X->buttons[n], style);
             create_kbd_accel(X->buttons[n], buttons[n].mods, buttons[n].value);
             gtk_widget_show(X->buttons[n]);
             gtk_table_attach(GTK_TABLE(X->ktable), X->buttons[n], 
@@ -1661,11 +1581,6 @@ new_cf_value(GtkMenuItem *item, gpointer user_data)
     gtk_entry_set_text(GTK_ENTRY(X->cf_desc_entry), "");
     gtk_entry_set_text(GTK_ENTRY(X->cf_val_entry), "");
  
-#ifdef FIXUP
-    XV_SET(Calctool_CFframe->CFpanel,
-           PANEL_CARET_ITEM, Calctool_CFframe->CFpi_cftext,
-           0);
-#endif /*FIXUP*/
     if (gdk_window_is_visible(X->cfframe->window) == FALSE) {
         ds_position_popup(X->kframe, X->cfframe, DS_POPUP_RIGHT);
     }
@@ -1771,13 +1686,9 @@ pframe_init(void)
 static void
 prop_apply(GtkButton *button, gpointer user_data)
 {
-    int oldr;
+    int newr = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(X->psright));
 
-    oldr          = v->righthand;
-    v->righthand  = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(X->psright));
-    set_prop_options(oldr);
-
-    write_cmdline();
+    set_prop_options(newr);
 }
 
 
@@ -1806,7 +1717,7 @@ prop_cancel(GtkButton *button, gpointer user_data)
 }
 
 
-/* Put calctool resource into deskset database. */
+/* Put gcalctool resource into deskset database. */
 
 void
 put_resource(enum res_type rtype, char *value)
@@ -1823,18 +1734,43 @@ put_resource(enum res_type rtype, char *value)
 
 
 static void
-reset_prop_vals(void)
+remove_extra_kbd_accels()
 {
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(X->psright), v->righthand);
+    remove_kbd_accel(BUT_EQ,   0,              GDK_Return);
+    remove_kbd_accel(BUT_MUL,  GDK_SHIFT_MASK, GDK_asterisk);
+    remove_kbd_accel(BUT_QUIT, GDK_SHIFT_MASK, GDK_Q);
+
+    /* Numeric keypad. */
+    remove_kbd_accel(BUT_0,   0, GDK_KP_0);
+    remove_kbd_accel(BUT_1,   0, GDK_KP_1);
+    remove_kbd_accel(BUT_2,   0, GDK_KP_2);
+    remove_kbd_accel(BUT_3,   0, GDK_KP_3);
+    remove_kbd_accel(BUT_4,   0, GDK_KP_4);
+    remove_kbd_accel(BUT_5,   0, GDK_KP_5);
+    remove_kbd_accel(BUT_6,   0, GDK_KP_6);
+    remove_kbd_accel(BUT_7,   0, GDK_KP_7);
+    remove_kbd_accel(BUT_8,   0, GDK_KP_8);
+    remove_kbd_accel(BUT_9,   0, GDK_KP_9);
+    remove_kbd_accel(BUT_ADD, 0, GDK_KP_Add);
+    remove_kbd_accel(BUT_SUB, 0, GDK_KP_Subtract);
+    remove_kbd_accel(BUT_MUL, 0, GDK_KP_Multiply);
+    remove_kbd_accel(BUT_DIV, 0, GDK_KP_Divide);
+    remove_kbd_accel(BUT_PNT, 0, GDK_KP_Delete);
+    remove_kbd_accel(BUT_EQ,  0, GDK_KP_Enter);
 }
 
 
-void
-save_cmdline(int argc, char *argv[])
+static void
+remove_kbd_accel(GtkWidget *button, guint button_mods, guint button_key) {
+    gtk_widget_remove_accelerator(button, X->kbd_accel, 
+                                  button_key, button_mods);
+}
+
+
+static void
+reset_prop_vals(void)
 {
-#ifdef FIXUP
-    DS_SAVE_CMDLINE(Calctool_kframe->kframe, argc, argv);
-#endif /*FIXUP*/
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(X->psright), v->righthand);
 }
 
 
@@ -1852,7 +1788,7 @@ save_resources()
 
     if ((filename = getenv("CALCTOOLDEFAULTS")) == NULL) {
         filename = (char *) calloc(1, strlen(home) + 18);
-        SPRINTF(filename, "%s/.calctooldefaults", home);
+        SPRINTF(filename, "%s/.gcalctooldefaults", home);
     }
 
 /* Check if file exists and user has write access. */
@@ -1900,18 +1836,20 @@ set_button_label(GtkWidget *button, gchar *str, int n)
 void
 set_button_state(enum fcp_type fcptype, int n, int isSensitive)
 {
-    int i;
-    GtkStyle *style;
     GtkWidget *w = (fcptype == FCP_KEY) ? X->buttons[n]: X->mode_buttons[n];
 
     gtk_widget_set_sensitive(w, isSensitive);
-    style = gtk_style_copy(gtk_widget_get_style(X->buttons[n]));
-    for (i = 0; i < 5; i++) {
-        style->bg[i] = (isSensitive ? X->palette[(int) buttons[n].color] :
-                                      X->palette[C_GREY]);
+}
+
+
+void
+set_help_state(int show_help)
+{
+    if (show_help) {
+        gtk_tooltips_enable(X->tips);
+    } else {
+        gtk_tooltips_disable(X->tips);
     }
-    gtk_widget_set_style(X->buttons[n], style);
-    gtk_widget_set_sensitive(w, isSensitive);
 }
 
 
@@ -1962,14 +1900,26 @@ set_mode(enum mode_type mode)
 
 
 static void
-set_prop_options(int oldr)
+set_prop_options(int newr)
 {
-    if (oldr != v->righthand) {
-        switch_hands(v->righthand);
+    int i, j, n;
+
+    if (newr != v->righthand) {
+        for (i = 0; i < BCOLS; i++) {
+            for (j = 0; j < BROWS; j++) {
+                n = j*BCOLS + i;
+                remove_kbd_accel(X->buttons[n], buttons[n].mods, 
+                                 buttons[n].value);
+            }
+        }
+        remove_extra_kbd_accels();
+        switch_hands(newr);
     }
 
+    v->righthand = newr;
     gtk_container_remove(GTK_CONTAINER(X->kvbox), X->ktable);
     make_ktable(X->kframe, X->kvbox); 
+    add_extra_kbd_accels();
     grey_buttons(v->base);
 }
 
@@ -2037,6 +1987,7 @@ void
 start_tool()
 {
     v->started = 1;
+    set_help_state(v->show_help);
     gtk_widget_show(X->kframe);
     gtk_main();
 }
